/******************************************************************************
 * defs.c
 *
 * Definition of basic functions and data types
 * 
 * Compile : gcc -o rw_tree main.c defs.c util.c
 * Usage   : rw_tree <input tree> <output tree>
 *
 ******************************************************************************
 * Copyright (C) 2016 José Fuentes Sepúlveda <jfuentess@dcc.uchile.cl>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *****************************************************************************/

typedef struct _edge_t Edge;
typedef struct _node_t Node;
typedef struct _tree_t Tree;
typedef struct _stack_t Stack;

typedef int item_t;

struct _stack_t {
  item_t *base;
  unsigned int top;
  unsigned int size;
};

// Used to represent graphs and trees
struct _edge_t {
  unsigned int src; // Index of the source vertex of the edge
  unsigned int tgt; // Index of the target vertex of the edge
  int cmp; // Position of the complementary edge (in the adjacency list of
	   // tgt). cmp < 0 means that the field cmp is undefined
};

// Used to represent trees
struct _node_t {
  unsigned int first; // Position of the first incident edge of a node in E
  unsigned int last; // Position of the last incident edge of a node in E
};

struct _tree_t {
  Node* N; // Array of nodes of the tree
  Edge* E; // Array of edges of the tree. It is the concatenation of the adjacency lists of all nodes
  unsigned int n; // Number of nodes in the tree
  // The number of edges is n-1
};

Stack *stack_create(unsigned int size);
int stack_empty(Stack *);
int stack_full(Stack *);
int stack_push(Stack *, item_t);
item_t stack_pop(Stack *);
item_t stack_top(Stack *);
void stack_free(Stack *);

int tree_cycle(Tree *);
