/******************************************************************************
 * util.c
 *
 * Implementation of the functions to read and write trees
 * 
 * Compile : gcc -o rw_tree main.c defs.c util.c
 * Usage   : rw_tree <input tree> <output tree>
 *
 ******************************************************************************
 * Copyright (C) 2016 José Fuentes Sepúlveda <jfuentess@dcc.uchile.cl>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "util.h"

Tree* read_tree_from_file(const char* fn) {
  Tree *t = malloc(sizeof(Tree));

  FILE *fp = fopen(fn, "r");
  char line_buffer[BUFSIZ]; /* BUFSIZ is defined if you include stdio.h */

  if (!fp) {
    fprintf(stderr, "Error opening file \"%s\".\n", fn);
    exit(EXIT_FAILURE);
  }

  t->n = atoi(fgets(line_buffer, sizeof(line_buffer), fp));

  t->N = calloc(t->n,sizeof(Node));
  t->E = calloc(2*(t->n-1),sizeof(Edge));

  unsigned int i = 0;
  for(i = 0; i < 2*(t->n-1); i++)
    t->E[i].cmp = -1;
  
  unsigned int source = 0, target = 0, m = 0;

  while (fgets(line_buffer, sizeof(line_buffer), fp)) {
    source = atoi(strtok(line_buffer, " "));
    target = atoi(strtok(NULL, " "));
    t->E[m].src = source;
    t->E[m].tgt = target;

    if(m==0)
      t->N[source].first = m;
    else if(source != t->E[m-1].src) {
      t->N[t->E[m-1].src].last = m-1;
      t->N[source].first = m;
    }
    m++;
  }
  t->N[t->E[m-1].src].last = m-1;

  fclose(fp);

  for(i = 0; i < 2*(t->n-1); i++) {
    Node target = t->N[t->E[i].tgt];
    int cmp = -1;

    if(t->E[i].cmp != -1)
      continue;
    
    unsigned int j = 0;
    for(j = target.first; j <= target.last; j++) {
      if((t->E[j].cmp == -1) && (t->E[j].tgt == t->E[i].src))
	cmp = j;
    }

    if(cmp != -1) {
      t->E[i].cmp = cmp;
      t->E[cmp].cmp = i;
    }
  }

  return t;
}


void write_tree_to_file(const char *fn, Tree *t) {

  FILE* fp = fopen(fn, "w");

  if (!fp) {
    fprintf(stderr, "Error opening file \"%s\".\n", fn);
    exit(EXIT_FAILURE);
  }

  unsigned int i = 0, j = 0;

  fprintf(fp, "%u\n", t->n);

  for(i = 0; i < t->n; i++)
    for(j = t->N[i].first; j <= t->N[i].last; j++)
      fprintf(fp, "%u %u\n", t->E[j].src, t->E[j].tgt);

  fclose(fp);
}
